import { BigNumber, Signer } from 'ethers';
import { BundleCommitted } from './events/BundleCommitted';
import { BundleForwarded } from './events/BundleForwarded';
import { BundleReceived } from './events/BundleReceived';
import { BundleSet } from './events/BundleSet';
import { ConfirmationSent } from './events/nft/ConfirmationSent';
import { EventFetcher } from './eventFetcher';
import { FeesSentToHub } from './events/FeesSentToHub';
import { MessageBundled } from './events/MessageBundled';
import { MessageExecuted } from './events/MessageExecuted';
import { MessageSent } from './events/MessageSent';
import { TokenConfirmed } from './events/nft/TokenConfirmed';
import { TokenSent } from './events/nft/TokenSent';
export type Options = {
    batchBlocks?: number;
    contractAddresses?: Record<string, any>;
};
export type BundleProof = {
    bundleId: string;
    treeIndex: number;
    siblings: string[];
    totalLeaves: number;
};
export type GetEventsInput = {
    chainId: number;
    fromBlock: number;
    toBlock?: number;
};
export type GetGeneralEventsInput = {
    eventName?: string;
    eventNames?: string[];
    chainId: number;
    fromBlock: number;
    toBlock?: number;
};
export type HasAuctionStartedInput = {
    fromChainId: number;
    bundleCommittedEvent: BundleCommitted;
};
export type GetSpokeExitTimeInput = {
    fromChainId: number;
    toChainId: number;
};
export type GetRelayRewardInput = {
    fromChainId: number;
    bundleCommittedEvent: BundleCommitted;
};
export type GetEstimatedTxCostForForwardMessageInput = {
    chainId: number;
};
export type ShouldAttemptForwardMessageInput = {
    fromChainId: number;
    bundleCommittedEvent: BundleCommitted;
};
export type GetBundleExitPopulatedTxInput = {
    fromChainId: number;
    bundleCommittedEvent?: BundleCommitted;
    bundleCommittedTransactionHash?: string;
};
export type ExitBundleInput = {
    fromChainId: number;
    bundleCommittedEvent?: BundleCommitted;
    bundleCommittedTransactionHash?: string;
    signer: Signer;
};
type GetIsL2TxHashExitedInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetSendMessagePopulatedTxInput = {
    fromChainId: number;
    toChainId: number;
    toAddress: string;
    toCalldata: string;
};
export type GetEventContextInput = {
    chainId: number;
    event: any;
};
export type GetRouteDataInput = {
    fromChainId: number;
    toChainId: number;
};
export type GetMessageFeeInput = {
    fromChainId: number;
    toChainId: number;
};
export type GetMaxBundleMessageCountInput = {
    fromChainId: number;
    toChainId: number;
};
export type GetIsBundleSetInput = {
    fromChainId: number;
    toChainId: number;
    bundleId: string;
};
export type GetMessageSentEventFromTransactionReceiptInput = {
    fromChainId: number;
    receipt: any;
};
export type GetMessageSentEventFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetMessageBundledEventFromMessageIdInput = {
    fromChainId: number;
    messageId: string;
};
export type GetMessageSentEventFromMessageIdInput = {
    fromChainId: number;
    messageId: string;
};
export type GetMessageExecutedEventFromMessageIdInput = {
    messageId: string;
    fromChainId: number;
    toChainId: number;
};
export type GetMessageBundledEventFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetMessageIdFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetMessageBundleIdFromMessageIdInput = {
    fromChainId: number;
    messageId: string;
};
export type GetMessageBundleIdFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetMessageTreeIndexFromMessageIdInput = {
    fromChainId: number;
    messageId: string;
};
export type GetMessageTreeIndexFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetMessageBundledEventsForBundleIdInput = {
    fromChainId: number;
    bundleId: string;
};
export type GetMessageIdsForBundleIdInput = {
    fromChainId: number;
    bundleId: string;
};
export type GetMerkleProofForMessageIdInput = {
    messageIds: string[];
    targetMessageId: string;
};
export type GetBundleProofFromMessageIdInput = {
    fromChainId: number;
    messageId: string;
};
export type GetBundleProofFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetRelayMessageDataFromTransactionHashInput = {
    fromChainId: number;
    transactionHash: string;
};
export type GetRelayMessagePopulatedTxInput = {
    fromChainId: number;
    toChainId: number;
    fromAddress: string;
    toAddress: string;
    toCalldata: string;
    bundleProof: BundleProof;
};
export type GetMessageCalldataInput = {
    fromChainId: number;
    messageId: string;
};
export type GetIsMessageIdRelayedInput = {
    messageId: string;
    fromChainId: number;
    toChainId: number;
};
export type GetNftMintPopulatedTxInput = {
    fromChainId: number;
    toAddress: string;
    tokenId: string;
};
export type GetNftBurnPopulatedTxInput = {
    fromChainId: number;
    tokenId: string;
};
export type GetNftSendPopulatedTxInput = {
    fromChainId: number;
    toChainId: number;
    toAddress: string;
    tokenId: string;
};
export type GetNftMintAndSendPopulatedTxInput = {
    fromChainId: number;
    toChainId: number;
    toAddress: string;
    tokenId: string;
};
export type GetNftConfirmPopulatedTxInput = {
    fromChainId: number;
    tokenId: string;
};
export type ConnectTargetsInput = {
    hubChainId: number;
    spokeChainId: number;
    target1: string;
    target2: string;
    signer: Signer;
};
export declare class Hop {
    eventFetcher: EventFetcher;
    network: string;
    batchBlocks?: number;
    contractAddresses: Record<string, any>;
    providers: Record<string, any>;
    l1ChainId: number;
    constructor(network?: string, options?: Options);
    get version(): string;
    getRpcProvider(chainId: number): any;
    setRpcProviders(providers: Record<string, any>): void;
    setRpcProvider(chainId: number, provider: any): void;
    getSpokeMessageBridgeContractAddress(chainId: number): string;
    getHubMessageBridgeContractAddress(chainId: number): string;
    getBundleCommittedEvents(input: GetEventsInput): Promise<BundleCommitted[]>;
    getBundleForwardedEvents(input: GetEventsInput): Promise<BundleForwarded[]>;
    getBundleReceivedEvents(input: GetEventsInput): Promise<BundleReceived[]>;
    getBundleSetEvents(input: GetEventsInput): Promise<BundleSet[]>;
    getFeesSentToHubEvents(input: GetEventsInput): Promise<FeesSentToHub[]>;
    getMessageBundledEvents(input: GetEventsInput): Promise<MessageBundled[]>;
    getMessageExecutedEvents(input: GetEventsInput): Promise<MessageExecuted[]>;
    getMessageSentEvents(input: GetEventsInput): Promise<MessageSent[]>;
    getEvents(input: GetGeneralEventsInput): Promise<any[]>;
    getEventNames(): string[];
    hasAuctionStarted(input: HasAuctionStartedInput): Promise<boolean>;
    getSpokeExitTime(input: GetSpokeExitTimeInput): Promise<number>;
    getRelayReward(input: GetRelayRewardInput): Promise<number>;
    getEstimatedTxCostForForwardMessage(input: GetEstimatedTxCostForForwardMessageInput): Promise<number>;
    shouldAttemptForwardMessage(input: ShouldAttemptForwardMessageInput): Promise<boolean>;
    getBundleExitPopulatedTx(input: GetBundleExitPopulatedTxInput): Promise<any>;
    exitBundle(input: ExitBundleInput): Promise<any>;
    getIsL2TxHashExited(input: GetIsL2TxHashExitedInput): Promise<any>;
    getSendMessagePopulatedTx(input: GetSendMessagePopulatedTxInput): Promise<any>;
    getBlock(chainId: number, blockNumber: number): Promise<any>;
    getChainSlug(chainId: number): string;
    getRelayWindowHours(): number;
    getRouteData(input: GetRouteDataInput): Promise<{
        messageFee: BigNumber;
        maxBundleMessages: number;
    }>;
    getMessageFee(input: GetMessageFeeInput): Promise<BigNumber>;
    getMaxBundleMessageCount(input: GetMaxBundleMessageCountInput): Promise<number>;
    private isValidTxHash;
    getContractAddresses(): any;
    setContractAddresses(contractAddresses: any): void;
    getIsBundleSet(input: GetIsBundleSetInput): Promise<boolean>;
    getMessageSentEventFromTransactionReceipt(input: GetMessageSentEventFromTransactionReceiptInput): Promise<MessageSent>;
    getMessageSentEventFromTransactionHash(input: GetMessageSentEventFromTransactionHashInput): Promise<MessageSent>;
    getMessageBundledEventFromMessageId(input: GetMessageBundledEventFromMessageIdInput): Promise<any>;
    getMessageSentEventFromMessageId(input: GetMessageSentEventFromMessageIdInput): Promise<any>;
    getMessageExecutedEventFromMessageId(input: GetMessageExecutedEventFromMessageIdInput): Promise<any>;
    getMessageBundledEventFromTransactionHash(input: GetMessageBundledEventFromTransactionHashInput): Promise<MessageBundled>;
    getMessageIdFromTransactionHash(input: GetMessageIdFromTransactionHashInput): Promise<string>;
    getMessageBundleIdFromMessageId(input: GetMessageBundleIdFromMessageIdInput): Promise<string>;
    getMessageBundleIdFromTransactionHash(input: GetMessageBundleIdFromTransactionHashInput): Promise<string>;
    getMessageTreeIndexFromMessageId(input: GetMessageTreeIndexFromMessageIdInput): Promise<number>;
    getMessageTreeIndexFromTransactionHash(input: GetMessageTreeIndexFromTransactionHashInput): Promise<number>;
    getMessageBundledEventsForBundleId(input: GetMessageBundledEventsForBundleIdInput): Promise<any[]>;
    getMessageIdsForBundleId(input: GetMessageIdsForBundleIdInput): Promise<string[]>;
    getMerkleProofForMessageId(input: GetMerkleProofForMessageIdInput): string[];
    getBundleProofFromMessageId(input: GetBundleProofFromMessageIdInput): Promise<BundleProof>;
    getBundleProofFromTransactionHash(input: GetBundleProofFromTransactionHashInput): Promise<BundleProof>;
    getRelayMessageDataFromTransactionHash(input: GetRelayMessageDataFromTransactionHashInput): Promise<{
        fromChainId: number;
        toAddress: string;
        fromAddress: string;
        toCalldata: string;
        toChainId: number;
        bundleProof: BundleProof;
    }>;
    getRelayMessagePopulatedTx(input: GetRelayMessagePopulatedTxInput): Promise<{
        chainId: number;
        to?: string;
        from?: string;
        nonce?: number;
        gasLimit?: BigNumber;
        gasPrice?: BigNumber;
        data?: string;
        value?: BigNumber;
        type?: number;
        accessList?: import("@ethersproject/transactions").AccessList;
        maxFeePerGas?: BigNumber;
        maxPriorityFeePerGas?: BigNumber;
        customData?: Record<string, any>;
        ccipReadEnabled?: boolean;
    }>;
    getSupportedChainIds(): number[];
    getMessageCalldata(input: GetMessageCalldataInput): Promise<string>;
    getIsMessageIdRelayed(input: GetIsMessageIdRelayedInput): Promise<boolean>;
    private isValidChainId;
    connectTargets(input: ConnectTargetsInput): Promise<any>;
    getNftConfirmationSentEvents(input: GetEventsInput): Promise<ConfirmationSent[]>;
    getNftTokenConfirmedEvents(input: GetEventsInput): Promise<TokenConfirmed[]>;
    getNftTokenSentEvents(input: GetEventsInput): Promise<TokenSent[]>;
    getNftBridgeContractAddress(chainId: number): string;
    getNftMintPopulatedTx(input: GetNftMintPopulatedTxInput): Promise<any>;
    getNftBurnPopulatedTx(input: GetNftBurnPopulatedTxInput): Promise<any>;
    getNftSendPopulatedTx(input: GetNftSendPopulatedTxInput): Promise<any>;
    getNftMintAndSendPopulatedTx(input: GetNftMintAndSendPopulatedTxInput): Promise<any>;
    getNftConfirmPopulatedTx(input: GetNftConfirmPopulatedTxInput): Promise<any>;
}
export {};
//# sourceMappingURL=Hop.d.ts.map