"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hop = void 0;
const package_json_1 = __importDefault(require("../package.json"));
const ethers_1 = require("ethers");
const BundleCommitted_1 = require("./events/BundleCommitted");
const BundleForwarded_1 = require("./events/BundleForwarded");
const BundleReceived_1 = require("./events/BundleReceived");
const BundleSet_1 = require("./events/BundleSet");
const ConfirmationSent_1 = require("./events/nft/ConfirmationSent");
const luxon_1 = require("luxon");
const ERC721Bridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/ERC721Bridge__factory");
const eventFetcher_1 = require("./eventFetcher");
const FeesSentToHub_1 = require("./events/FeesSentToHub");
const HubERC5164ConnectorFactory__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/HubERC5164ConnectorFactory__factory");
const HubMessageBridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/HubMessageBridge__factory");
const MerkleTree_1 = require("./utils/MerkleTree");
const MessageBundled_1 = require("./events/MessageBundled");
const MessageExecuted_1 = require("./events/MessageExecuted");
const MessageSent_1 = require("./events/MessageSent");
const SpokeMessageBridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/SpokeMessageBridge__factory");
const TokenConfirmed_1 = require("./events/nft/TokenConfirmed");
const TokenSent_1 = require("./events/nft/TokenSent");
const chainSlugMap_1 = require("./utils/chainSlugMap");
const utils_1 = require("ethers/lib/utils");
const getProvider_1 = require("./utils/getProvider");
const addresses_1 = require("@hop-protocol/v2-core/addresses");
const cache = {};
class Hop {
    constructor(network = 'goerli', options) {
        this.contractAddresses = {
            mainnet: {},
            goerli: addresses_1.goerliAddresses
        };
        this.providers = {};
        if (!['mainnet', 'goerli'].includes(network)) {
            throw new Error(`Invalid network: ${network}`);
        }
        this.network = network;
        if (this.network === 'mainnet') {
            this.l1ChainId = 1;
        }
        else if (this.network === 'goerli') {
            this.l1ChainId = 5;
        }
        if (options === null || options === void 0 ? void 0 : options.batchBlocks) {
            this.batchBlocks = options.batchBlocks;
        }
        if (options === null || options === void 0 ? void 0 : options.contractAddresses) {
            this.contractAddresses[network] = options.contractAddresses;
        }
    }
    get version() {
        return package_json_1.default.version;
    }
    getRpcProvider(chainId) {
        if (this.providers[chainId]) {
            return this.providers[chainId];
        }
        return (0, getProvider_1.getProvider)(this.network, chainId);
    }
    setRpcProviders(providers) {
        for (const chainId in providers) {
            this.setRpcProvider(Number(chainId), providers[chainId]);
        }
    }
    setRpcProvider(chainId, provider) {
        if (typeof provider === 'string') {
            provider = new ethers_1.providers.StaticJsonRpcProvider(provider);
        }
        this.providers[chainId] = provider;
    }
    getSpokeMessageBridgeContractAddress(chainId) {
        var _a, _b;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        const address = (_b = (_a = this.contractAddresses[this.network]) === null || _a === void 0 ? void 0 : _a[chainId]) === null || _b === void 0 ? void 0 : _b.spokeCoreMessenger;
        return address;
    }
    getHubMessageBridgeContractAddress(chainId) {
        var _a, _b;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        const address = (_b = (_a = this.contractAddresses[this.network]) === null || _a === void 0 ? void 0 : _a[chainId]) === null || _b === void 0 ? void 0 : _b.hubCoreMessenger;
        return address;
    }
    async getBundleCommittedEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new BundleCommitted_1.BundleCommittedEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getBundleForwardedEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getHubMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new BundleForwarded_1.BundleForwardedEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getBundleReceivedEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getHubMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new BundleReceived_1.BundleReceivedEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getBundleSetEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new BundleSet_1.BundleSetEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getFeesSentToHubEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new FeesSentToHub_1.FeesSentToHubEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getMessageBundledEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new MessageBundled_1.MessageBundledEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getMessageExecutedEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new MessageExecuted_1.MessageExecutedEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getMessageSentEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new MessageSent_1.MessageSentEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getEvents(input) {
        let { eventName, eventNames, chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const latestBlock = await provider.getBlockNumber();
        if (latestBlock) {
            if (!toBlock) {
                toBlock = latestBlock;
            }
            if (!fromBlock) {
                const start = latestBlock - 1000;
                fromBlock = start;
            }
            if (fromBlock < 0) {
                fromBlock = toBlock + fromBlock;
            }
        }
        if (eventName) {
            eventNames = [eventName];
        }
        if (!eventNames.length) {
            throw new Error('expected eventName or eventNames');
        }
        const filters = [];
        const eventFetcher = new eventFetcher_1.EventFetcher({
            provider,
            batchBlocks: this.batchBlocks
        });
        const map = {};
        for (const eventName of eventNames) {
            if (eventName === 'BundleCommitted') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new BundleCommitted_1.BundleCommittedEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'BundleForwared') {
                const address = this.getHubMessageBridgeContractAddress(chainId);
                const _eventFetcher = new BundleForwarded_1.BundleForwardedEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'BundleReceived') {
                const address = this.getHubMessageBridgeContractAddress(chainId);
                const _eventFetcher = new BundleReceived_1.BundleReceivedEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'BundleSet') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new BundleSet_1.BundleSetEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'FeesSentToHub') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new FeesSentToHub_1.FeesSentToHubEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'MessageBundled') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new MessageBundled_1.MessageBundledEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'MessageExecuted') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new MessageExecuted_1.MessageExecutedEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'MessageSent') {
                const address = this.getSpokeMessageBridgeContractAddress(chainId);
                const _eventFetcher = new MessageSent_1.MessageSentEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'ConfirmationSent') { // nft
                const address = this.getNftBridgeContractAddress(chainId);
                const _eventFetcher = new ConfirmationSent_1.ConfirmationSentEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'TokenConfirmed') { // nft
                const address = this.getNftBridgeContractAddress(chainId);
                const _eventFetcher = new TokenConfirmed_1.TokenConfirmedEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
            else if (eventName === 'TokenSent') { // nft
                const address = this.getNftBridgeContractAddress(chainId);
                const _eventFetcher = new TokenSent_1.TokenSentEventFetcher(provider, chainId, this.batchBlocks, address);
                const filter = _eventFetcher.getFilter();
                filters.push(filter);
                map[filter.topics[0]] = _eventFetcher;
            }
        }
        const options = {
            fromBlock,
            toBlock
        };
        const events = await eventFetcher.fetchEvents(filters, options);
        const decoded = [];
        for (const event of events) {
            const res = await map[event.topics[0]].populateEvents([event]);
            decoded.push(...res);
        }
        return decoded;
    }
    getEventNames() {
        return [
            'BundleCommitted',
            'BundleForwarded',
            'BundleReceived',
            'BundleSet',
            'FeesSentToHub',
            'MessageBundled',
            'MessageExecuted',
            'MessageSent'
        ];
    }
    async hasAuctionStarted(input) {
        const { fromChainId, bundleCommittedEvent } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const { commitTime, toChainId } = bundleCommittedEvent;
        const exitTime = await this.getSpokeExitTime({ fromChainId, toChainId });
        return commitTime + exitTime < luxon_1.DateTime.utc().toSeconds();
    }
    async getSpokeExitTime(input) {
        const { fromChainId, toChainId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        const provider = this.getRpcProvider(toChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${toChainId}`);
        }
        const address = this.getHubMessageBridgeContractAddress(toChainId);
        if (!address) {
            throw new Error(`Invalid chain: ${toChainId}`);
        }
        const hubMessageBridge = HubMessageBridge__factory_1.HubMessageBridge__factory.connect(address, provider);
        const exitTime = await hubMessageBridge.getSpokeExitTime(fromChainId);
        const exitTimeSeconds = Number(exitTime.toString());
        return exitTimeSeconds;
    }
    // relayReward = (block.timestamp - relayWindowStart) * feesCollected / relayWindow
    // reference: https://github.com/hop-protocol/contracts-v2/blob/master/contracts/bridge/FeeDistributor/FeeDistributor.sol#L83-L106
    async getRelayReward(input) {
        const { fromChainId, bundleCommittedEvent } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const { commitTime, bundleFees, toChainId } = bundleCommittedEvent;
        const feesCollected = Number((0, utils_1.formatEther)(bundleFees));
        const { timestamp: blockTimestamp } = await provider.getBlock('latest');
        const spokeExitTime = await this.getSpokeExitTime({ fromChainId, toChainId });
        const relayWindowStart = commitTime + spokeExitTime;
        const relayWindow = this.getRelayWindowHours() * 60 * 60;
        const relayReward = (blockTimestamp - relayWindowStart) * feesCollected / relayWindow;
        return relayReward;
    }
    async getEstimatedTxCostForForwardMessage(input) {
        const { chainId } = input;
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${chainId}`);
        }
        const estimatedGas = ethers_1.BigNumber.from(1000000); // TODO
        const gasPrice = await provider.getGasPrice();
        const estimatedTxCost = estimatedGas.mul(gasPrice);
        return Number((0, utils_1.formatUnits)(estimatedTxCost, 9));
    }
    async shouldAttemptForwardMessage(input) {
        const { fromChainId, bundleCommittedEvent } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const estimatedTxCost = await this.getEstimatedTxCostForForwardMessage({ chainId: fromChainId });
        const relayReward = await this.getRelayReward({ fromChainId, bundleCommittedEvent });
        const txOk = relayReward > estimatedTxCost;
        const timeOk = await this.hasAuctionStarted({ fromChainId, bundleCommittedEvent });
        const shouldAttempt = txOk && timeOk;
        return shouldAttempt;
    }
    async getBundleExitPopulatedTx(input) {
        var _a;
        let { fromChainId, bundleCommittedEvent, bundleCommittedTransactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (bundleCommittedTransactionHash) {
            if (!this.isValidTxHash(bundleCommittedTransactionHash)) {
                throw new Error(`Invalid transaction hash: ${bundleCommittedTransactionHash}`);
            }
        }
        else if (bundleCommittedEvent) {
            const { eventLog, context } = bundleCommittedEvent;
            bundleCommittedTransactionHash = (_a = eventLog.transactionHash) !== null && _a !== void 0 ? _a : context === null || context === void 0 ? void 0 : context.transactionHash;
        }
        if (!bundleCommittedTransactionHash) {
            throw new Error('expected bundle comitted transaction hash');
        }
        const l1Provider = this.getRpcProvider(this.l1ChainId);
        const l2Provider = this.getRpcProvider(fromChainId);
        let exitRelayer;
        if ([420, 10].includes(fromChainId)) {
            const { OptimismRelayer } = await Promise.resolve().then(() => __importStar(require('./exitRelayers/OptimismRelayer')));
            exitRelayer = new OptimismRelayer(this.network, l1Provider, l2Provider);
        }
        else if ([421613, 42161, 42170].includes(fromChainId)) {
            // const { ArbitrumRelayer } = await import('./exitRelayers/ArbitrumRelayer')
            // exitRelayer = new ArbitrumRelayer(this.network, l1Provider, l2Provider)
        }
        else if ([80001, 137].includes(fromChainId)) {
            // const { PolygonRelayer } = await import('./exitRelayers/PolygonRelayer')
            // exitRelayer = new PolygonRelayer(this.network, l1Provider, l2Provider)
        }
        else if ([100].includes(fromChainId)) {
            // const { GnosisChainRelayer } = await import('./exitRelayers/GnosisChainRelayer')
            // exitRelayer = new GnosisChainRelayer(this.network, l1Provider, l2Provider)
        }
        if (!exitRelayer) {
            throw new Error(`Exit relayer not found for chainId "${fromChainId}"`);
        }
        const txData = await exitRelayer.getExitPopulatedTx(bundleCommittedTransactionHash);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
    async exitBundle(input) {
        var _a;
        let { fromChainId, bundleCommittedEvent, bundleCommittedTransactionHash, signer } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (bundleCommittedTransactionHash) {
            if (!this.isValidTxHash(bundleCommittedTransactionHash)) {
                throw new Error(`Invalid transaction hash: ${bundleCommittedTransactionHash}`);
            }
        }
        else if (bundleCommittedEvent) {
            const { eventLog, context } = bundleCommittedEvent;
            bundleCommittedTransactionHash = (_a = eventLog.transactionHash) !== null && _a !== void 0 ? _a : context === null || context === void 0 ? void 0 : context.transactionHash;
        }
        if (!bundleCommittedTransactionHash) {
            throw new Error('expected bundle comitted transaction hash');
        }
        const l1Provider = this.getRpcProvider(this.l1ChainId);
        const l2Provider = this.getRpcProvider(fromChainId);
        let exitRelayer;
        if ([420, 10].includes(fromChainId)) {
            const { OptimismRelayer } = await Promise.resolve().then(() => __importStar(require('./exitRelayers/OptimismRelayer')));
            exitRelayer = new OptimismRelayer(this.network, signer, l2Provider);
        }
        else if ([421613, 42161, 42170].includes(fromChainId)) {
            // const { ArbitrumRelayer } = await import('./exitRelayers/ArbitrumRelayer')
            // exitRelayer = new ArbitrumRelayer(this.network, l1Provider, l2Provider)
        }
        else if ([80001, 137].includes(fromChainId)) {
            // const { PolygonRelayer } = await import('./exitRelayers/PolygonRelayer')
            // exitRelayer = new PolygonRelayer(this.network, l1Provider, l2Provider)
        }
        else if ([100].includes(fromChainId)) {
            // const { GnosisChainRelayer } = await import('./exitRelayers/GnosisChainRelayer')
            // exitRelayer = new GnosisChainRelayer(this.network, l1Provider, l2Provider)
        }
        if (!exitRelayer) {
            throw new Error(`Exit relayer not found for chainId "${fromChainId}"`);
        }
        const tx = await exitRelayer.exitTx(bundleCommittedTransactionHash);
        return tx;
    }
    async getIsL2TxHashExited(input) {
        const { fromChainId, transactionHash } = input;
        const l1Provider = this.getRpcProvider(this.l1ChainId);
        const l2Provider = this.getRpcProvider(fromChainId);
        let exitRelayer;
        if ([420, 10].includes(fromChainId)) {
            const { OptimismRelayer } = await Promise.resolve().then(() => __importStar(require('./exitRelayers/OptimismRelayer')));
            exitRelayer = new OptimismRelayer(this.network, l1Provider, l2Provider);
        }
        if (!exitRelayer) {
            throw new Error(`Exit relayer not found for chainId "${fromChainId}"`);
        }
        return exitRelayer.getIsL2TxHashExited(transactionHash);
    }
    async getSendMessagePopulatedTx(input) {
        let { fromChainId, toChainId, toAddress, toCalldata = '0x' } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        if (!toAddress) {
            throw new Error('toAddress is required');
        }
        if (!toCalldata) {
            toCalldata = '0x';
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Invalid address: ${fromChainId}`);
        }
        const spokeMessageBridge = SpokeMessageBridge__factory_1.SpokeMessageBridge__factory.connect(address, provider);
        const txData = await spokeMessageBridge.populateTransaction.dispatchMessage(toChainId, toAddress, toCalldata);
        const value = await this.getMessageFee({ fromChainId, toChainId });
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId, value: value.toString() });
    }
    async getBlock(chainId, blockNumber) {
        const cacheKey = `${chainId}-${blockNumber}`;
        if (cache[cacheKey]) {
            return cache[cacheKey];
        }
        const provider = this.getRpcProvider(chainId);
        const block = await provider.getBlock(blockNumber);
        cache[cacheKey] = block;
        return block;
    }
    getChainSlug(chainId) {
        const chainSlug = chainSlugMap_1.chainSlugMap[chainId];
        if (!chainSlug) {
            throw new Error(`Invalid chain: ${chainId}`);
        }
        return chainSlug;
    }
    // reference: https://github.com/hop-protocol/contracts-v2/blob/cdc3377d6a1f964554ba0e6e1fef0b504d43fc6a/contracts/bridge/FeeDistributor/FeeDistributor.sol#L42
    getRelayWindowHours() {
        return 12;
    }
    async getRouteData(input) {
        const { fromChainId, toChainId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        const provider = this.getRpcProvider(fromChainId);
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        const spokeMessageBridge = SpokeMessageBridge__factory_1.SpokeMessageBridge__factory.connect(address, provider);
        const routeData = await spokeMessageBridge.routeData(toChainId);
        return {
            messageFee: routeData.messageFee,
            maxBundleMessages: Number(routeData.maxBundleMessages.toString())
        };
    }
    async getMessageFee(input) {
        const { fromChainId, toChainId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        const routeData = await this.getRouteData({ fromChainId, toChainId });
        return routeData.messageFee;
    }
    async getMaxBundleMessageCount(input) {
        const { fromChainId, toChainId } = input;
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        const routeData = await this.getRouteData({ fromChainId, toChainId });
        return routeData.maxBundleMessages;
    }
    isValidTxHash(txHash) {
        return txHash.slice(0, 2) === '0x' && txHash.length === 66;
    }
    getContractAddresses() {
        return this.contractAddresses[this.network];
    }
    setContractAddresses(contractAddresses) {
        this.contractAddresses[this.network] = contractAddresses;
    }
    async getIsBundleSet(input) {
        const { fromChainId, toChainId, bundleId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        const provider = this.getRpcProvider(toChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${toChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(toChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${toChainId}`);
        }
        const hubMessageBridge = HubMessageBridge__factory_1.HubMessageBridge__factory.connect(address, provider);
        const entity = await hubMessageBridge.bundles(bundleId);
        if (!entity) {
            return false;
        }
        return ethers_1.BigNumber.from(entity.root).gt(0) && Number(entity.fromChainId.toString()) === fromChainId;
    }
    async getMessageSentEventFromTransactionReceipt(input) {
        const { fromChainId, receipt } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${fromChainId}`);
        }
        const eventFetcher = new MessageSent_1.MessageSentEventFetcher(provider, fromChainId, this.batchBlocks, address);
        const filter = eventFetcher.getFilter();
        for (const log of receipt.logs) {
            if (log.topics[0] === filter.topics[0]) {
                const decoded = eventFetcher.toTypedEvent(log);
                return decoded;
            }
        }
        return null;
    }
    async getMessageSentEventFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const receipt = await provider.getTransactionReceipt(transactionHash);
        return this.getMessageSentEventFromTransactionReceipt({ fromChainId, receipt });
    }
    async getMessageBundledEventFromMessageId(input) {
        var _a;
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${fromChainId}`);
        }
        const eventFetcher = new MessageBundled_1.MessageBundledEventFetcher(provider, fromChainId, 1000000000, address);
        const filter = eventFetcher.getMessageIdFilter(messageId);
        const toBlock = await provider.getBlockNumber();
        const fromBlock = 0; // endBlock - 100_000
        const events = await eventFetcher._getEvents(filter, fromBlock, toBlock);
        return (_a = events === null || events === void 0 ? void 0 : events[0]) !== null && _a !== void 0 ? _a : null;
    }
    async getMessageSentEventFromMessageId(input) {
        var _a;
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!fromChainId) {
            throw new Error('fromChainId is required');
        }
        if (!messageId) {
            throw new Error('messageId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${fromChainId}`);
        }
        const eventFetcher = new MessageSent_1.MessageSentEventFetcher(provider, fromChainId, 1000000000, address);
        const filter = eventFetcher.getMessageIdFilter(messageId);
        const toBlock = await provider.getBlockNumber();
        const fromBlock = 0; // endBlock - 100_000
        const events = await eventFetcher._getEvents(filter, fromBlock, toBlock);
        return (_a = events === null || events === void 0 ? void 0 : events[0]) !== null && _a !== void 0 ? _a : null;
    }
    // note: this is broken because messageId is not indexed in event
    async getMessageExecutedEventFromMessageId(input) {
        var _a;
        const { fromChainId, toChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (!messageId) {
            throw new Error('messageId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${toChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(toChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${toChainId}`);
        }
        const eventFetcher = new MessageExecuted_1.MessageExecutedEventFetcher(provider, toChainId, 1000000000, address);
        const filter = eventFetcher.getMessageIdFilter(messageId);
        const toBlock = await provider.getBlockNumber();
        const fromBlock = 0; // endBlock - 100_000
        const events = await eventFetcher._getEvents(filter, fromBlock, toBlock);
        return (_a = events === null || events === void 0 ? void 0 : events[0]) !== null && _a !== void 0 ? _a : null;
    }
    async getMessageBundledEventFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const receipt = await provider.getTransactionReceipt(transactionHash);
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${fromChainId}`);
        }
        const eventFetcher = new MessageBundled_1.MessageBundledEventFetcher(provider, fromChainId, this.batchBlocks, address);
        const filter = eventFetcher.getFilter();
        for (const log of receipt.logs) {
            if (log.topics[0] === filter.topics[0]) {
                const decoded = eventFetcher.toTypedEvent(log);
                return decoded;
            }
        }
        return null;
    }
    async getMessageIdFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!fromChainId) {
            throw new Error('fromChainId is required');
        }
        if (!transactionHash) {
            throw new Error('transactionHash is required');
        }
        const event = await this.getMessageSentEventFromTransactionHash({ fromChainId, transactionHash });
        if (!event) {
            throw new Error('event not found for transaction hash');
        }
        return event.messageId;
    }
    async getMessageBundleIdFromMessageId(input) {
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const event = await this.getMessageBundledEventFromMessageId({ fromChainId, messageId });
        if (!event) {
            throw new Error('event not found for messageId');
        }
        return event.bundleId;
    }
    async getMessageBundleIdFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        const event = await this.getMessageBundledEventFromTransactionHash({ fromChainId, transactionHash });
        if (!event) {
            throw new Error('event not found for transaction hash');
        }
        return event.bundleId;
    }
    async getMessageTreeIndexFromMessageId(input) {
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const event = await this.getMessageBundledEventFromMessageId({ fromChainId, messageId });
        if (!event) {
            throw new Error('event not found for messageId');
        }
        return event.treeIndex;
    }
    async getMessageTreeIndexFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const event = await this.getMessageBundledEventFromTransactionHash({ fromChainId, transactionHash });
        if (!event) {
            throw new Error('event not found for transaction hash');
        }
        return event.treeIndex;
    }
    async getMessageBundledEventsForBundleId(input) {
        const { fromChainId, bundleId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        const address = this.getSpokeMessageBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${fromChainId}`);
        }
        const eventFetcher = new MessageBundled_1.MessageBundledEventFetcher(provider, fromChainId, 1000000000, address);
        const filter = eventFetcher.getBundleIdFilter(bundleId);
        const toBlock = await provider.getBlockNumber();
        const fromBlock = 0; // endBlock - 100_000
        const events = eventFetcher._getEvents(filter, fromBlock, toBlock);
        return events;
    }
    async getMessageIdsForBundleId(input) {
        const { fromChainId, bundleId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const messageEvents = await this.getMessageBundledEventsForBundleId({ fromChainId, bundleId });
        const messageIds = messageEvents.map((item) => item.messageId);
        return messageIds;
    }
    getMerkleProofForMessageId(input) {
        const { messageIds, targetMessageId } = input;
        if (!targetMessageId) {
            throw new Error('targetMessageId is required');
        }
        const tree = MerkleTree_1.MerkleTree.from(messageIds);
        const proof = tree.getHexProof(targetMessageId);
        return proof;
    }
    async getBundleProofFromMessageId(input) {
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        if (!messageId) {
            throw new Error('messageId is required');
        }
        const { treeIndex, bundleId } = await this.getMessageBundledEventFromMessageId({ fromChainId, messageId });
        const messageIds = await this.getMessageIdsForBundleId({ fromChainId, bundleId });
        const siblings = this.getMerkleProofForMessageId({ messageIds, targetMessageId: messageId });
        const totalLeaves = messageIds.length;
        return {
            bundleId,
            treeIndex,
            siblings,
            totalLeaves
        };
    }
    async getBundleProofFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${fromChainId}`);
        }
        // TODO: handle case for when multiple message events in single transaction
        const { treeIndex, bundleId } = await this.getMessageBundledEventFromTransactionHash({ fromChainId, transactionHash });
        const targetMessageId = await this.getMessageIdFromTransactionHash({ fromChainId, transactionHash });
        const messageIds = await this.getMessageIdsForBundleId({ fromChainId, bundleId });
        const siblings = this.getMerkleProofForMessageId({ messageIds, targetMessageId });
        const totalLeaves = messageIds.length;
        return {
            bundleId,
            treeIndex,
            siblings,
            totalLeaves
        };
    }
    async getRelayMessageDataFromTransactionHash(input) {
        const { fromChainId, transactionHash } = input;
        const event = await this.getMessageSentEventFromTransactionHash({ fromChainId, transactionHash });
        const toAddress = event.to;
        const fromAddress = event.from;
        const toCalldata = event.data;
        const toChainId = event.toChainId;
        const bundleProof = await this.getBundleProofFromTransactionHash({ fromChainId, transactionHash });
        return {
            fromChainId,
            toAddress,
            fromAddress,
            toCalldata,
            toChainId,
            bundleProof
        };
    }
    async getRelayMessagePopulatedTx(input) {
        const { fromChainId, toChainId, fromAddress, toAddress, toCalldata, bundleProof } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        const provider = this.getRpcProvider(toChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${toChainId}`);
        }
        const address = this.getHubMessageBridgeContractAddress(toChainId);
        if (!address) {
            throw new Error(`Invalid chain: ${toChainId}`);
        }
        const hubMessageBridge = HubMessageBridge__factory_1.HubMessageBridge__factory.connect(address, provider);
        const txData = await hubMessageBridge.populateTransaction.executeMessage(fromChainId, fromAddress, toAddress, toCalldata, bundleProof);
        return Object.assign(Object.assign({}, txData), { chainId: toChainId });
    }
    getSupportedChainIds() {
        const keys = Object.keys(this.contractAddresses[this.network]);
        return keys.map((chainId) => Number(chainId));
    }
    async getMessageCalldata(input) {
        const { fromChainId, messageId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!messageId) {
            throw new Error('messageId is required');
        }
        const event = await this.getMessageSentEventFromMessageId({ fromChainId, messageId });
        if (!event) {
            throw new Error(`Event not found for messageId: ${messageId}`);
        }
        return event.data;
    }
    async getIsMessageIdRelayed(input) {
        const { messageId, fromChainId, toChainId } = input;
        if (!messageId) {
            throw new Error('messageId is required');
        }
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        const event = await this.getMessageExecutedEventFromMessageId({ messageId, fromChainId, toChainId });
        return !!event;
    }
    isValidChainId(chainId) {
        if (!chainId) {
            throw new Error('chainId is required');
        }
        const chainIds = new Set(Object.keys(this.contractAddresses[this.network]).map((chainId) => Number(chainId)));
        return chainIds.has(chainId);
    }
    async connectTargets(input) {
        var _a, _b, _c, _d;
        const { hubChainId, spokeChainId, target1, target2, signer } = input;
        const provider = this.getRpcProvider(hubChainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${hubChainId}`);
        }
        const address = (_b = (_a = this.contractAddresses[this.network]) === null || _a === void 0 ? void 0 : _a[hubChainId]) === null || _b === void 0 ? void 0 : _b.hubConnectorFactory;
        if (!address) {
            throw new Error('address not found for hub connector factory');
        }
        const factory = HubERC5164ConnectorFactory__factory_1.HubERC5164ConnectorFactory__factory.connect(address, signer);
        const tx = await factory.connectTargets(hubChainId, target1, spokeChainId, target2);
        const receipt = await tx.wait();
        const event = (_c = receipt.events) === null || _c === void 0 ? void 0 : _c.find((event) => event.event === 'ConnectorDeployed');
        const connectorAddress = (0, utils_1.getAddress)((_d = event === null || event === void 0 ? void 0 : event.args) === null || _d === void 0 ? void 0 : _d.connector);
        return { connectorAddress };
    }
    // nft start ////////////////////////////////////////////////////////////////
    async getNftConfirmationSentEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getNftBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new ConfirmationSent_1.ConfirmationSentEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getNftTokenConfirmedEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getNftBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new TokenConfirmed_1.TokenConfirmedEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    async getNftTokenSentEvents(input) {
        const { chainId, fromBlock, toBlock } = input;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        if (!fromBlock) {
            throw new Error('fromBlock is required');
        }
        const provider = this.getRpcProvider(chainId);
        if (!provider) {
            throw new Error(`Provider not found for chainId: ${chainId}`);
        }
        const address = this.getNftBridgeContractAddress(chainId);
        if (!address) {
            throw new Error(`Contract address not found for chainId: ${chainId}`);
        }
        const eventFetcher = new TokenSent_1.TokenSentEventFetcher(provider, chainId, this.batchBlocks, address);
        return eventFetcher.getEvents(fromBlock, toBlock);
    }
    getNftBridgeContractAddress(chainId) {
        var _a, _b;
        if (!chainId) {
            throw new Error('chainId is required');
        }
        const address = (_b = (_a = this.contractAddresses[this.network]) === null || _a === void 0 ? void 0 : _a[chainId]) === null || _b === void 0 ? void 0 : _b.nftBridge;
        return address;
    }
    async getNftMintPopulatedTx(input) {
        const { fromChainId, toAddress, tokenId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!toAddress) {
            throw new Error('toAddress is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getNftBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Nft bridge address not found for chainId "${fromChainId}"`);
        }
        const nftBridge = ERC721Bridge__factory_1.ERC721Bridge__factory.connect(address, provider);
        const txData = await nftBridge.populateTransaction.mint(toAddress, tokenId);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
    async getNftBurnPopulatedTx(input) {
        const { fromChainId, tokenId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!tokenId) {
            throw new Error('tokenId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getNftBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Invalid address: ${fromChainId}`);
        }
        const nftBridge = ERC721Bridge__factory_1.ERC721Bridge__factory.connect(address, provider);
        const txData = await nftBridge.populateTransaction.burn(tokenId);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
    async getNftSendPopulatedTx(input) {
        const { fromChainId, toChainId, toAddress, tokenId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        if (!toAddress) {
            throw new Error('toAddress is required');
        }
        if (!tokenId) {
            throw new Error('tokenId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getNftBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Invalid address: ${fromChainId}`);
        }
        const nftBridge = ERC721Bridge__factory_1.ERC721Bridge__factory.connect(address, provider);
        const txData = await nftBridge.populateTransaction.send(toChainId, toAddress, tokenId);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
    async getNftMintAndSendPopulatedTx(input) {
        const { fromChainId, toChainId, toAddress, tokenId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!this.isValidChainId(toChainId)) {
            throw new Error(`Invalid toChainId: ${toChainId}`);
        }
        if (fromChainId === toChainId) {
            throw new Error('fromChainId and toChainId must be different');
        }
        if (!toAddress) {
            throw new Error('toAddress is required');
        }
        if (!tokenId) {
            throw new Error('tokenId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getNftBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Invalid address: ${fromChainId}`);
        }
        const nftBridge = ERC721Bridge__factory_1.ERC721Bridge__factory.connect(address, provider);
        const txData = await nftBridge.populateTransaction.mintAndSend(toChainId, toAddress, tokenId);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
    async getNftConfirmPopulatedTx(input) {
        const { fromChainId, tokenId } = input;
        if (!this.isValidChainId(fromChainId)) {
            throw new Error(`Invalid fromChainId: ${fromChainId}`);
        }
        if (!tokenId) {
            throw new Error('tokenId is required');
        }
        const provider = this.getRpcProvider(fromChainId);
        if (!provider) {
            throw new Error(`Invalid chain: ${fromChainId}`);
        }
        const address = this.getNftBridgeContractAddress(fromChainId);
        if (!address) {
            throw new Error(`Invalid address: ${fromChainId}`);
        }
        const nftBridge = ERC721Bridge__factory_1.ERC721Bridge__factory.connect(address, provider);
        const txData = await nftBridge.populateTransaction.confirm(tokenId);
        return Object.assign(Object.assign({}, txData), { chainId: fromChainId });
    }
}
exports.Hop = Hop;
