"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Nft = void 0;
const utils_1 = require("ethers/lib/utils");
const getProvider_1 = require("./utils/getProvider");
class Nft {
    constructor(network = 'goerli') {
        this.populateTransaction = {
            getRpcProvider(chainId) {
                return (0, getProvider_1.getProvider)('goerli', chainId);
            },
            mintNft(input) {
                const { contractAddress, fromChainId, recipient, tokenId } = input;
                const ABI = [
                    'function safeMint(address to, uint256 tokenId)'
                ];
                const iface = new utils_1.Interface(ABI);
                const data = iface.encodeFunctionData('safeMint', [recipient, tokenId]);
                const txData = {
                    to: contractAddress,
                    data
                };
                return txData;
            },
            approveNft(input) {
                const { contractAddress, fromChainId, spender, tokenId } = input;
                const ABI = [
                    'function approve(address spender, uint256 tokenId)'
                ];
                const iface = new utils_1.Interface(ABI);
                const data = iface.encodeFunctionData('approve', [spender, tokenId]);
                const txData = {
                    to: contractAddress,
                    data
                };
                return txData;
            }
        };
        this.network = network;
    }
    mintNftWrapper(input) {
        const { nftBridgeAddress, wrapperTokenId, fromChainId, serialNumber, supportedChains, wrapperTokenIdNonce } = input;
        const ABI = [
            'function mintWrapper(uint256 wrapperTokenId, uint256 fromChainId, bytes32 serialNumber, uint256[] memory supportedChainIds, uint256 wrapperTokenIdNonce) public'
        ];
        const iface = new utils_1.Interface(ABI);
        const data = iface.encodeFunctionData('mintWrapper', [wrapperTokenId, fromChainId, serialNumber, supportedChains, wrapperTokenIdNonce]);
        const txData = {
            to: nftBridgeAddress,
            data
        };
        return txData;
    }
    reclaimNftWrapper(input) {
        const { nftBridgeAddress, nftTokenAddress, tokenId, fromChainId, serialNumber, supportedChainIds, wrapperTokenIdNonce } = input;
        const ABI = [
            'function mintWrapperAndWithdraw(address nftTokenAddress, uint256 nftTokenId, uint256 fromChainId, bytes32 serialNumber, uint256[] memory supportedChainIds, uint256 wrapperTokenIdNonce) public'
        ];
        const iface = new utils_1.Interface(ABI);
        const data = iface.encodeFunctionData('mintWrapperAndWithdraw', [nftTokenAddress, tokenId, fromChainId, serialNumber, supportedChainIds, wrapperTokenIdNonce]);
        const txData = {
            to: nftBridgeAddress,
            data
        };
        return txData;
    }
    sendNft(input) {
        const { nftBridgeAddress, fromChainId, contractAddress, tokenId, supportedChainIds, toChainId, recipient, wrapperTokenIdNonce } = input;
        const ABI = [
            'function depositAndAttemptMintAndSend(address nftTokenAddress, uint256 nftTokenId, uint256[] memory supportedChainIds, uint256 toChainId, address recipient, uint256 wrapperTokenIdNonce) public returns (uint256, bytes32)'
        ];
        const iface = new utils_1.Interface(ABI);
        const args = [contractAddress, tokenId, supportedChainIds, toChainId, recipient, wrapperTokenIdNonce];
        const data = iface.encodeFunctionData('depositAndAttemptMintAndSend', args);
        const txData = {
            to: nftBridgeAddress,
            data
        };
        return txData;
    }
    sendNftWrapper(input) {
        const { nftBridgeAddress, wrapperTokenId, fromChainId, serialNumber, supportedChainIds, initialRecipient, toChainId, recipient, wrapperTokenIdNonce } = input;
        const ABI = [
            'function send(uint256 wrapperTokenId, uint256 fromChainId, bytes32 serialNumber, uint256[] memory supportedChainIds, address initialRecipient, uint256 toChainId, address recipient, uint256 wrapperTokenIdNonce) public'
        ];
        const iface = new utils_1.Interface(ABI);
        const data = iface.encodeFunctionData('send', [wrapperTokenId, fromChainId, serialNumber, supportedChainIds, initialRecipient, toChainId, recipient, wrapperTokenIdNonce]);
        const value = '1000000000000';
        const txData = {
            to: nftBridgeAddress,
            data,
            value
        };
        return txData;
    }
}
exports.Nft = Nft;
