"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BundleCommittedEventFetcher = void 0;
const SpokeMessageBridge_json_1 = __importDefault(require("@hop-protocol/v2-core/abi/generated/SpokeMessageBridge.json"));
const ethers_1 = require("ethers");
const Event_1 = require("./Event");
const SpokeMessageBridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/SpokeMessageBridge__factory");
class BundleCommittedEventFetcher extends Event_1.Event {
    constructor() {
        super(...arguments);
        this.eventName = 'BundleCommitted';
    }
    getFilter() {
        const spokeMessageBridge = SpokeMessageBridge__factory_1.SpokeMessageBridge__factory.connect(this.address, this.provider);
        const filter = spokeMessageBridge.filters.BundleCommitted();
        return filter;
    }
    async getEvents(startBlock, endBlock) {
        const filter = this.getFilter();
        return this._getEvents(filter, startBlock, endBlock);
    }
    toTypedEvent(ethersEvent) {
        const iface = new ethers_1.ethers.utils.Interface(SpokeMessageBridge_json_1.default);
        const decoded = iface.parseLog(ethersEvent);
        const bundleId = decoded.args.bundleId.toString();
        const bundleRoot = decoded.args.bundleRoot.toString();
        const bundleFees = decoded.args.bundleFees;
        const toChainId = Number(decoded.args.toChainId.toString());
        const commitTime = Number(decoded.args.commitTime.toString());
        return {
            eventName: this.eventName,
            eventLog: ethersEvent,
            bundleId,
            bundleRoot,
            bundleFees,
            toChainId,
            commitTime
        };
    }
}
exports.BundleCommittedEventFetcher = BundleCommittedEventFetcher;
