"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BundleReceivedEventFetcher = void 0;
const HubMessageBridge_json_1 = __importDefault(require("@hop-protocol/v2-core/abi/generated/HubMessageBridge.json"));
const ethers_1 = require("ethers");
const Event_1 = require("./Event");
const HubMessageBridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/HubMessageBridge__factory");
class BundleReceivedEventFetcher extends Event_1.Event {
    constructor() {
        super(...arguments);
        this.eventName = 'BundleReceived';
    }
    getFilter() {
        const hubMessageBridge = HubMessageBridge__factory_1.HubMessageBridge__factory.connect(this.address, this.provider);
        const filter = hubMessageBridge.filters.BundleReceived();
        return filter;
    }
    async getEvents(startBlock, endBlock) {
        const filter = this.getFilter();
        return this._getEvents(filter, startBlock, endBlock);
    }
    toTypedEvent(ethersEvent) {
        const iface = new ethers_1.ethers.utils.Interface(HubMessageBridge_json_1.default);
        const decoded = iface.parseLog(ethersEvent);
        const bundleId = decoded.args.bundleId.toString();
        const bundleRoot = decoded.args.bundleRoot.toString();
        const bundleFees = decoded.args.bundleFees;
        const fromChainId = Number(decoded.args.fromChainId.toString());
        const toChainId = Number(decoded.args.toChainId.toString());
        const relayWindowStart = Number(decoded.args.relayWindowStart.toString());
        const relayer = decoded.args.relayer.toString();
        return {
            eventName: this.eventName,
            eventLog: ethersEvent,
            bundleId,
            bundleRoot,
            bundleFees,
            fromChainId,
            toChainId,
            relayWindowStart,
            relayer
        };
    }
}
exports.BundleReceivedEventFetcher = BundleReceivedEventFetcher;
