"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSentEventFetcher = void 0;
const SpokeMessageBridge_json_1 = __importDefault(require("@hop-protocol/v2-core/abi/generated/SpokeMessageBridge.json"));
const Event_1 = require("./Event");
const SpokeMessageBridge__factory_1 = require("@hop-protocol/v2-core/contracts/factories/generated/SpokeMessageBridge__factory");
const ethers_1 = require("ethers");
class MessageSentEventFetcher extends Event_1.Event {
    constructor() {
        super(...arguments);
        this.eventName = 'MessageSent';
    }
    getFilter() {
        const spokeMessageBridge = SpokeMessageBridge__factory_1.SpokeMessageBridge__factory.connect(this.address, this.provider);
        const filter = spokeMessageBridge.filters.MessageSent();
        return filter;
    }
    getMessageIdFilter(messageId) {
        const spokeMessageBridge = SpokeMessageBridge__factory_1.SpokeMessageBridge__factory.connect(this.address, this.provider);
        const filter = spokeMessageBridge.filters.MessageSent(messageId);
        return filter;
    }
    async getEvents(startBlock, endBlock) {
        const filter = this.getFilter();
        return this._getEvents(filter, startBlock, endBlock);
    }
    toTypedEvent(ethersEvent) {
        const iface = new ethers_1.ethers.utils.Interface(SpokeMessageBridge_json_1.default);
        const decoded = iface.parseLog(ethersEvent);
        const messageId = decoded.args.messageId.toString();
        const from = decoded.args.from;
        const toChainId = Number(decoded.args.toChainId.toString());
        const to = decoded.args.to;
        const data = decoded.args.data;
        return {
            eventName: this.eventName,
            eventLog: ethersEvent,
            messageId,
            from,
            toChainId,
            to,
            data
        };
    }
}
exports.MessageSentEventFetcher = MessageSentEventFetcher;
