"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriceFeed = void 0;
const CoinGecko_1 = require("./CoinGecko");
const Coinbase_1 = require("./Coinbase");
class PriceFeed {
    constructor() {
        this.services = [new CoinGecko_1.CoinGecko(), new Coinbase_1.Coinbase()];
        this.cacheTimeMs = 5 * 60 * 1000;
        this.cache = {};
        this.aliases = {
            WETH: 'ETH',
            WMATIC: 'MATIC',
            XDAI: 'DAI',
            WXDAI: 'DAI'
        };
    }
    async getPriceByTokenSymbol(tokenSymbol) {
        if (this.aliases[tokenSymbol]) {
            tokenSymbol = this.aliases[tokenSymbol];
        }
        const cached = this.cache[tokenSymbol];
        if (cached) {
            const isRecent = cached.timestamp > Date.now() - this.cacheTimeMs;
            if (isRecent) {
                return cached.price;
            }
        }
        console.log(`fetching new token price for ${tokenSymbol}`);
        const errors = [];
        for (const service of this.services) {
            try {
                const price = await service.getPriceByTokenSymbol(tokenSymbol);
                if (price === null) {
                    throw new Error(`null price for ${tokenSymbol}`);
                }
                this.cache[tokenSymbol] = {
                    timestamp: Date.now(),
                    price
                };
                console.log(`fetched new token price of ${price} for ${tokenSymbol}`);
                return price;
            }
            catch (err) {
                console.debug(`error fetching price for ${tokenSymbol}`);
                const isLastService = this.services.indexOf(service) === this.services.length - 1;
                errors.push(err.message);
                if (isLastService) {
                    throw new Error(`PriceFeed error(s): ${errors.join(' ')}`);
                }
            }
        }
    }
}
exports.PriceFeed = PriceFeed;
