"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.promiseQueue = void 0;
const p_queue_1 = __importDefault(require("p-queue"));
const lodash_1 = require("lodash");
async function promiseQueue(items, cb, options) {
    return promiseQueueConcurrent(items, cb, options);
}
exports.promiseQueue = promiseQueue;
async function promiseQueueChunk(items, cb, options) {
    const { concurrency } = options;
    const allChunks = (0, lodash_1.chunk)(items, concurrency);
    let i = 0;
    for (const chunks of allChunks) {
        await Promise.all(chunks.map(async (item) => {
            i++;
            await cb(item, i);
        }));
    }
}
async function promiseQueueConcurrent(items, cb, options) {
    const { concurrency, timeoutMs: timeout } = options;
    const queue = new p_queue_1.default({ concurrency, timeout });
    for (let i = 0; i < items.length; i++) {
        queue.add(async () => await cb(items[i], i));
    }
    await queue.onEmpty();
    await queue.onIdle();
}
